import json

from django.conf.urls import patterns, url
from django.contrib import admin
from django.core.files.storage import default_storage
from django.http import Http404, HttpResponse, HttpResponseBadRequest

from .helpers import save_file


class QuillAdmin(admin.ModelAdmin):

    """Admin class offering the ability to accept file uploads."""

    def get_urls(self):
        """Add URLs needed to handle image uploads."""
        urls = patterns(
            '',
            url(r'^upload/$', self.admin_site.admin_view(self.handle_upload), name='quill-file-upload'),
        )
        return urls + super(QuillAdmin, self).get_urls()

    def handle_upload(self, request):
        """Handle file uploads from WYSIWYG."""
        if request.method != 'POST':
            raise Http404

        if request.is_ajax():
            try:
                filename = request.GET['quillUploadFile']
                data = request
                is_raw = True
            except KeyError:
                return HttpResponseBadRequest("Invalid file upload.")
        else:
            if len(request.FILES) != 1:
                return HttpResponseBadRequest("Can only upload 1 file at a time.")
            try:
                data = request.FILES['quillUploadFile']
                filename = data.name
                is_raw = False
            except KeyError:
                return HttpResponseBadRequest('Missing image `quillUploadFile`.')

        url = save_file(data, filename, is_raw, default_storage)
        response_data = {}
        response_data['url'] = url

        # Response content type needs to be text/html here or else
        # IE will try to download the file.
        return HttpResponse(json.dumps(response_data), content_type="text/html; charset=utf-8")



from django.apps import AppConfig


class QuillConfig(AppConfig):

    """Base configuration for django-quill."""

    name = 'quill'

    editor_selector = 'editor'
    toolbar_selector = 'toolbar'
    theme = 'snow'
    allowed_image_extensions = ['jpg', 'jpeg', 'png', 'gif']

    full = {
        'editor_template': 'quill/editor.html',
        'toolbar_template': 'quill/toolbars/full.html',
        'template': 'quill/widget.html',
    }
    basic = dict(full, toolbar_template='quill/toolbars/basic.html')
    default = full



from django import forms
from django.apps import apps
from django.forms.util import flatatt
from django.template.loader import render_to_string
from django.templatetags.static import static
from django.utils.safestring import mark_safe


class QuillEditorWidget(forms.Textarea):

    """Widget used to render a QuillJS WYSIWYG."""

    class Media:
        css = {
            'all': (
                static('quill/css/vendor/quill-0.19.11.snow.css'),
                static('quill/css/quill.css'),
            )
        }

        js = (
            static('quill/js/vendor/SimpleAjaxUploader-1a6f62289d.min.js'),
            static('quill/js/build/quill-django.min.js'),
        )

    def __init__(self, config='default', *args, **kwargs):
        """Create a new Quill WYSIWYG Widget.

        :param str config: The QuillJS config to use (from :py:class:`quill.apps.QuillConfig`)

        """
        self.config = config
        super(QuillEditorWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs={}):
        """Render the Quill WYSIWYG."""
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        quill_app = apps.get_app_config('quill')
        quill_config = getattr(quill_app, self.config)

        return mark_safe(render_to_string(quill_config['template'], {
            'final_attrs': flatatt(final_attrs),
            'value': value,
            'id': final_attrs['id'],
            'config': self.config,
        }))



from django.db import models
from django import forms

from .widgets import QuillEditorWidget


class RichTextField(models.TextField):

    """Extended TextField that provides WYSIWYG with QuillJS."""

    def __init__(self, config='default', *args, **kwargs):
        """Create a new WYSIWYG field.

        :param str config: The QuillJS config to use (from :py:class:`quill.apps.QuillConfig`)

        """
        self.config = config
        super(RichTextField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """Get the form for field."""
        defaults = {
            'form_class': RichTextFormField,
            'config': self.config,
        }
        defaults.update(kwargs)
        return super(RichTextField, self).formfield(**defaults)


class RichTextFormField(forms.fields.CharField):

    """Extend form field for QuillJS WYSIWYG."""

    def __init__(self, config='default', *args, **kwargs):
        """Create a new WYSIWYG form field.

        :param str config: The QuillJS config to use (from :py:class:`quill.apps.QuillConfig`)

        """
        kwargs.update({
            'widget': QuillEditorWidget(config=config)
        })
        super(RichTextFormField, self).__init__(*args, **kwargs)



import os
import sys

from setuptools import setup

if sys.argv[-1] == 'publish':
    os.system('make build')
    os.system('python setup.py sdist upload')
    sys.exit()

import quill

with open('README.md', 'r') as readme_file:
    readme = readme_file.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-quill',
    version=quill.__version__,
    author='Ryan Senkbeil',
    author_email='ryan.senkbeil@gsdesign.com',
    description='Easily use Quill.js in your django admin.',
    long_description=readme,
    packages=['quill'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
